package com.zanxiang.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.manage.domain.dto.GameTagDTO;
import com.zanxiang.manage.domain.params.GameTagAddUpdateParam;
import com.zanxiang.manage.domain.params.GameTagListParam;
import com.zanxiang.manage.domain.vo.GameTagChoiceVO;
import com.zanxiang.manage.domain.vo.GameTagVO;
import com.zanxiang.mybatis.entity.GameTag;

import java.util.List;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-11-16
 * @description : 游戏分类标签
 */
public interface GameTagService extends IService<GameTag> {

    /**
     * 根据主键id查询分类以及子分类
     *
     * @param ids : 主键id列表
     * @return : 返回游戏分类信息
     */
    List<GameTagVO> listByIds(List<Long> ids);

    /**
     * 查询所有标签
     *
     * @return : 返回所有游戏标签
     */
    List<GameTagChoiceVO> gameTagsChoiceList();

    /**
     * 查询游戏分类标签
     *
     * @return : 返回所有游戏分类和标签
     */
    Map<Long, GameTagDTO> gameTagMap();

    /**
     * 查询游戏标签列表
     *
     * @param param : 游戏标签列表查询呢参数
     * @return : 返回分页信息
     */
    IPage<GameTagVO> list(GameTagListParam param);

    /**
     * 游戏标签分类添加或修改
     *
     * @param param : 参数
     * @return : 返回结果
     */
    Boolean addOrUpdate(GameTagAddUpdateParam param);

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return : 删除结果
     */
    Boolean deleteById(Long id);
}
