package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.GameCategoryDTO;
import com.zanxiang.manage.domain.vo.GameCategoryChoiceVO;
import com.zanxiang.manage.service.GameCategoryService;
import com.zanxiang.mybatis.entity.GameCategory;
import com.zanxiang.mybatis.mapper.GameCategoryMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-23
 * @description : 游戏应用类型
 */
@Slf4j
@Service
public class GameCategoryServiceImpl extends ServiceImpl<GameCategoryMapper, GameCategory> implements GameCategoryService {

    /**
     * 根据主键id查询
     *
     * @param id : 主键id
     * @return : 返回游戏分类信息
     */
    @Override
    public GameCategoryDTO getById(Long id) {
        GameCategory gameCategory = super.getById(id);
        if (gameCategory == null) {
            throw new BaseException("参数错误, 游戏应用类型不存在");
        }
        return BeanUtils.copy(gameCategory, GameCategoryDTO.class);
    }

    /**
     * 查询所有应用类型列表
     *
     * @return : 返回所有应用类型
     */
    @Override
    public List<GameCategoryChoiceVO> gameCategoryChoiceList() {
        return BeanUtils.copyList(super.list(), GameCategoryChoiceVO.class);
    }

    /**
     * 查询游戏分类和标签
     *
     * @return : 返回所有游戏分类和标签
     */
    @Override
    public Map<Long, GameCategoryDTO> gameCategoryMap() {
        List<GameCategory> gameCategoryList = list(new LambdaQueryWrapper<GameCategory>()
                .select(GameCategory::getId, GameCategory::getName));
        if (CollectionUtils.isEmpty(gameCategoryList)) {
            return Collections.emptyMap();
        }
        List<GameCategoryDTO> gameCategoryDTOList = BeanUtils.copyList(gameCategoryList, GameCategoryDTO.class);
        return gameCategoryDTOList.stream().collect(Collectors.toMap(GameCategoryDTO::getId, Function.identity()));
    }

}
