package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.manage.domain.params.PromoAccountAssignParam;
import com.zanxiang.manage.service.PromoAccountLogService;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.mybatis.entity.PromoAccount;
import com.zanxiang.mybatis.entity.PromoAccountLog;
import com.zanxiang.mybatis.mapper.PromoAccountLogMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * @author : lingfeng
 * @time : 2022-11-22
 * @description : 推广账号指派日志
 */
@Slf4j
@Service
public class PromoAccountLogServiceImpl extends ServiceImpl<PromoAccountLogMapper, PromoAccountLog> implements PromoAccountLogService {

    /**
     * 创建推广账号日志
     *
     * @param promoAccount : 推广账号
     * @return : 返回创建结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean create(PromoAccount promoAccount) {
        LocalDateTime dateTime = LocalDateTime.now();
        return super.save(PromoAccountLog.builder()
                .accountId(promoAccount.getId())
                .pitcherId(promoAccount.getPitcherId())
                .pitcherGroupId(promoAccount.getPitcherGroupId())
                .startTime(dateTime)
                .createTime(dateTime)
                .updateTime(dateTime)
                .build());
    }

    /**
     * 账号指派更新
     *
     * @param param : 推广账号指派参数
     * @return : 返回更新结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean assignUpdate(PromoAccountAssignParam param) {
        super.update(new LambdaUpdateWrapper<PromoAccountLog>()
                .set(PromoAccountLog::getEndTime, LocalDateTime.now())
                .eq(PromoAccountLog::getAccountId, param.getId()));
        return super.save(PromoAccountLog.builder()
                .accountId(param.getId())
                .pitcherId(param.getPitcherId())
                .pitcherGroupId(param.getPitcherGroupId())
                .startTime(DateUtil.localDateToLocalDateTime(param.getStartTime()))
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build());
    }
}
