package com.zanxiang.sdk.service;


import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.sdk.domain.params.*;
import com.zanxiang.sdk.domain.vo.UserLoginVO;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author : lingfeng
 * @time : 2022-06-22
 * @description :
 */
public interface IRegisterLoginService {

    /**
     * 微信小游戏code授权登录
     *
     * @param param    : 授权参数
     * @param userData : 用户信息
     * @return {@link ResultVO}<{@link UserLoginVO}>
     */
    ResultVO<UserLoginVO> loginWxCode(LoginVxCodeParam param, UserData userData);

    /**
     * 登录密码
     * 用户名注册或者登录
     *
     * @param param    : 注册登录参数
     * @param userData : 用户信息
     * @return {@link ResultVO}<{@link UserLoginVO}>
     */
    ResultVO<UserLoginVO> loginPassword(LoginPasswordParam param, UserData userData);

    /**
     * 手机号登录
     *
     * @param param    : 手机号登录参数
     * @param userData : 用户信息
     * @return {@link ResultVO}<{@link UserLoginVO}>
     */
    ResultVO<UserLoginVO> loginMobile(LoginMobileParam param, UserData userData);

    /**
     * 登录退出
     *
     * @param userData : 返回用户信息
     * @return {@link Boolean}
     */
    Boolean loginOut(UserData userData);

    /**
     * 登录注销
     *
     * @param userData : 返回用户信息
     * @return {@link Boolean}
     */
    Boolean writeOff(UserData userData);

    /**
     * QQ授权登录
     *
     * @param response : 返回体
     */
    void qqLoginAuth(HttpServletResponse response);

    /**
     * QQ授权注册登录回调
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @param userData             : userData
     * @return {@link ResultVO}<{@link UserLoginVO}>
     */
    ResultVO<UserLoginVO> qqLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request, UserData userData);

    /**
     * 微信授权登录
     *
     * @param response 响应
     */
    void vxLoginAuth(HttpServletResponse response);

    /**
     * 微信授权注册登录回调
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @param userData             : userData
     * @return {@link ResultVO}<{@link UserLoginVO}>
     */
    ResultVO<UserLoginVO> vxLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request, UserData userData);
}
