package com.zanxiang.sdk.service.pay;

import com.alibaba.fastjson.JSONObject;
import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.alipay.easysdk.kernel.util.ResponseChecker;
import com.alipay.easysdk.payment.app.models.AlipayTradeAppPayResponse;
import com.alipay.easysdk.payment.common.models.AlipayTradeCloseResponse;
import com.alipay.easysdk.payment.page.models.AlipayTradePagePayResponse;
import com.alipay.easysdk.payment.wap.models.AlipayTradeWapPayResponse;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.enums.OsEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.dto.GamePayWayDTO;
import com.zanxiang.sdk.domain.dto.PayApplicationDTO;
import com.zanxiang.sdk.domain.dto.PayMerchantDTO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.service.IPayApplicationService;
import com.zanxiang.sdk.service.IPayMerchantService;
import com.zanxiang.sdk.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author xufeng
 * @date 2022/6/8 15:37
 */
@Slf4j
@Service
public class AliPayService extends PayBaseService {

    private Pattern pattern = Pattern.compile("action=\"(.*)\">", Pattern.CASE_INSENSITIVE);

    private Pattern param = Pattern.compile("value=\"(.*)\">", Pattern.CASE_INSENSITIVE);

    private Pattern mobileClientParam = Pattern.compile("\\{\"queryResultUrl\".*?\"}", Pattern.CASE_INSENSITIVE);

    /**
     * 异步回调地址配置
     */
    @Value("${payConfig.aliPay.returnUrl}")
    private String returnUrlConfig;

    /**
     * 同步回调地址配置
     */
    @Value("${payConfig.aliPay.notifyUrl}")
    private String notifyUrlConfig;

    /**
     * 异步回调地址
     */
    private String returnUrl;

    /**
     * 同步回调地址
     */
    private String notifyUrl;

    /**
     * 支付配置
     */
    private Config config;

    /**
     * 商户信息
     */
    private PayMerchantDTO payMerchant;

    @Autowired
    private IPayMerchantService payMerchantService;

    @Autowired
    private IPayApplicationService payApplicationService;

    @Override
    public Map<String, Object> create(ProductPayParamBO product, GamePayWayDTO gamePayWayDTO) {
//        this.attach = product.getAttach();
        log.error("支付宝支付, attach : {}", JsonUtil.toString(this.attach));
        this.configInit(gamePayWayDTO);
        //回调地址中拼接游戏具体支付方式
        this.returnUrl = this.returnUrlConfig + "?" + "gamePayWayId=" + attach.getGamePayWayId();
        this.notifyUrl = this.notifyUrlConfig + "?" + "gamePayWayId=" + attach.getGamePayWayId();
        //支付方式
        int payDevice = product.getPayDevice().intValue();
        //不同的支付途径
        Map<String, Object> resultMap;
        switch (payDevice) {
            case 1:
                resultMap = this.pcPay(product);
                break;
            case 2:
                resultMap = this.h5Pay(product);
                break;
            case 3:
                resultMap = this.appPay(product);
                break;
            default:
                throw new RuntimeException("未知支付方式");
        }
        //更新订单商户信息
        orderService.update(new LambdaUpdateWrapper<Order>()
                .set(Order::getMerchantNo, payMerchant.getMerchantNo())
                .set(Order::getMerchantName, payMerchant.getMerchantName())
                .set(Order::getStatus, OrderStateEnum.WAIT_PAY.getCode())
                .eq(Order::getOrderId, product.getOutTradeNo()));
        //返回结果
        return resultMap;
    }

    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) {
        try {
            Map<String, String> params = new HashMap<>();
            Enumeration<String> parameterNames = request.getParameterNames();
            while (parameterNames.hasMoreElements()) {
                String parameterName = parameterNames.nextElement();
                params.put(parameterName, request.getParameter(parameterName));
            }
            String orderNo = request.getParameter("out_trade_no");
            String gamePayWayId = params.get("gamePayWayId");
            //回调地址拼接参数不参加sign验证
            params.remove("gamePayWayId");
            //初始化配置
            configInit(gamePayWayService.getById(Long.valueOf(gamePayWayId)));
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                if (paySuccess(orderNo, params.get("total_amount"), params.get("trade_no"))) {
                    log.info("支付宝异步回调成功, orderId : {}", orderNo);
                    return HttpStatusEnum.SUCCESS.getMsg();
                }
            }
            log.error("支付宝异步回调-验证签名失败 request:{}", request);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return HttpStatusEnum.FAIL.getMsg();
    }

    @Override
    public ResultVO synNotify(HttpServletRequest request) {
        try {
            //获取支付宝GET过来反馈信息
            Map<String, String> params = new HashMap<>();
            Map<String, String[]> requestParams = request.getParameterMap();
            for (String name : requestParams.keySet()) {
                String[] values = requestParams.get(name);
                String valueStr = "";
                for (int i = 0; i < values.length; i++) {
                    valueStr = (i == values.length - 1) ? valueStr + values[i]
                            : valueStr + values[i] + ",";
                }
                //乱码解决，这段代码在出现乱码时使用
                valueStr = new String(valueStr.getBytes(StandardCharsets.ISO_8859_1), StandardCharsets.UTF_8);
                params.put(name, valueStr);
            }
            //商户订单号
            String orderNo = new String(request.getParameter("out_trade_no").getBytes(StandardCharsets.ISO_8859_1), StandardCharsets.UTF_8);
            String gamePayWayId = params.get("gamePayWayId");
            //回调地址拼接参数不参加sign验证
            params.remove("gamePayWayId");
            //初始化配置
            configInit(gamePayWayService.getById(Long.valueOf(gamePayWayId)));
            //验证签名
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                if (paySuccess(orderNo, params.get("total_amount"), params.get("trade_no"))) {
                    log.info("支付宝同步回调成功, orderId : {}", orderNo);
                    return ResultVO.ok();
                }
            }
            log.error("支付宝同步回调验证签名结果失败, orderId : {}", orderNo);
            return ResultVO.fail(HttpStatusEnum.FAIL.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            //处理异常信息
            return ResultVO.fail(e.getMessage());
        }
    }

    @Override
    public void closeOrder(PlatformOrderDTO platformOrderDTO) {
        //支付配置
        GamePayWayDTO gamePayWayDTO = gamePayWayService.getGamePayWay(platformOrderDTO.getGameId(),
                platformOrderDTO.getPayWayId(), platformOrderDTO.getPayDeviceId());
        //初始化配置
        this.configInit(gamePayWayDTO);
        //取消订单
        AlipayTradeCloseResponse response;
        try {
            response = Factory.Payment.Common().close(platformOrderDTO.getOrderId());
            if (ResponseChecker.success(response)) {
                return;
            }
            log.error("支付宝取消订单失败, platformOrderDTO : {}, response : {}", JsonUtil.toString(platformOrderDTO), JsonUtil.toString(response));
        } catch (Exception e) {
            log.error("支付宝取消订单异常, platformOrderDTO : {}, e:{}", JsonUtil.toString(platformOrderDTO), e.getMessage());
        }
    }

    private Map<String, Object> h5Pay(ProductPayParamBO product) {
        AlipayTradeWapPayResponse response;
        try {
            response = Factory.Payment.Wap().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), "", this.returnUrl);
        } catch (Exception e) {
            log.error("支付宝H5支付异常, product : {}, e:{}", JsonUtil.toString(product), e.getMessage());
            throw new BaseException("支付宝H5支付异常");
        }
        //请求失败
        if (!ResponseChecker.success(response)) {
            log.error("支付宝H5支付失败, product : {}, response : {}", JsonUtil.toString(product), JsonUtil.toString(response));
            throw new BaseException("支付宝H5支付失败");
        }
        //结果数据
        String body = response.getBody();
        Map<String, Object> payParamMap = new HashMap<>(3);
        payParamMap.put("deviceType", product.getPayDevice());
        payParamMap.put("orderId", product.getOutTradeNo());
        //非手机系统, 不走deepLink
        if (!product.getDeviceSystem().contains(OsEnum.SYSTEM_ANDROID.getOs())
                && !product.getDeviceSystem().contains(OsEnum.SYSTEM_IOS.getOs())) {
            payParamMap.put("data", body);
            return payParamMap;
        }
        //解析短链deepLink
        String deepLink = this.getH5DeepLink(body, product.getDeviceSystem());
        payParamMap.put("data", deepLink);
        return payParamMap;
    }

    private String getH5DeepLink(String body, String deviceSystem) {
        Matcher matcher = pattern.matcher(body);
        if (!matcher.find()) {
            return null;
        }
        Matcher paraMatcher = param.matcher(body);
        if (!paraMatcher.find()) {
            return null;
        }
        String url = matcher.group();
        url = url.substring(8, url.length() - 2);
        String paramData = paraMatcher.group();
        paramData = paramData.substring(7, paramData.length() - 2);
        paramData = paramData.replace("&quot;", "\"");
        String mobileClientUrl = HttpUtil.postDataUrl(url, "biz_content=" + paramData, null);
        String aliPayContentBody = HttpUtil.postData(mobileClientUrl, "");
        Matcher mobileClientMatcher = mobileClientParam.matcher(aliPayContentBody);
        if (!mobileClientMatcher.find()) {
            return null;
        }
        String deepLinkAndroid = "alipays://platformapi/startApp?appId=20000125&orderSuffix=";
        String deepLinkIos = "alipay://alipayclient/?";
        String pageData = mobileClientMatcher.group() + "}";
        JSONObject jsonObject = JSONObject.parseObject(pageData);
        Object invokeAliPayData = jsonObject.get("invokeAlipayData");
        String dataString = JSONObject.parseObject(invokeAliPayData.toString()).get("dataString").toString();
        try {
            deepLinkAndroid += URLEncoder.encode(dataString, Constants.UTF8);
            deepLinkIos += URLEncoder.encode(pageData, Constants.UTF8);
        } catch (Exception e) {
            log.error("deepLink短链encode异常, e : {}", e.getMessage());
            throw new BaseException("deepLink短链encode异常");
        }
        return Objects.equals(deviceSystem, OsEnum.SYSTEM_ANDROID.getOs()) ? deepLinkAndroid : deepLinkIos;
    }

    private Map<String, Object> pcPay(ProductPayParamBO product) {
        AlipayTradePagePayResponse response;
        try {
            response = Factory.Payment.Page().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), this.returnUrl);
        } catch (Exception e) {
            log.error("支付宝pc端支付异常, product : {}, e:{}", JsonUtil.toString(product), e.getMessage());
            throw new BaseException("支付宝pc端支付异常");
        }
        if (!ResponseChecker.success(response)) {
            log.error("支付宝pc端支付失败, product : {}, response : {}", JsonUtil.toString(product), JsonUtil.toString(response));
            throw new BaseException("支付宝pc端支付失败");
        }
        Map<String, Object> payParamMap = new HashMap<>(3);
        payParamMap.put("deviceType", product.getPayDevice());
        payParamMap.put("data", response.getBody());
        payParamMap.put("orderId", product.getOutTradeNo());
        return payParamMap;
    }

    private Map<String, Object> appPay(ProductPayParamBO product) {
        AlipayTradeAppPayResponse response;
        try {
            response = Factory.Payment.App().asyncNotify(notifyUrl)
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee());
        } catch (Exception e) {
            log.error("支付宝app端支付异常, product : {}, e:{}", JsonUtil.toString(product), e.getMessage());
            throw new BaseException("支付宝app端支付异常");
        }
        if (!ResponseChecker.success(response)) {
            log.error("支付宝app端支付失败, product : {}, response : {}", JsonUtil.toString(product), JsonUtil.toString(response));
            throw new BaseException("支付宝app端支付失败");
        }
        Map<String, Object> payParamMap = new HashMap<>(3);
        payParamMap.put("deviceType", product.getPayDevice());
        payParamMap.put("data", response.getBody());
        payParamMap.put("orderId", product.getOutTradeNo());
        return payParamMap;
    }

    private void configInit(GamePayWayDTO gamePayWayDTO) {
        //商户信息
        PayMerchantDTO payMerchantDTO = payMerchantService.getByMerchantNo(gamePayWayDTO.getMerchantNo());
        this.payMerchant = payMerchantDTO;
        //查询支付应用信息
        PayApplicationDTO payApplicationDTO = payApplicationService.getPayApplicationByAppId(gamePayWayDTO.getAppId());
        //支付参数
        Map<String, String> payConfigMap = JsonUtil.toMap(payMerchantDTO.getPayConfig(), Map.class, String.class);
        //支付宝应用id和应用私钥
        payConfigMap.put("appId", payApplicationDTO.getAppId());
        payConfigMap.put("merchantPrivateKey", payApplicationDTO.getPrivateKey());
        //赋值配置信息
        this.config = JsonUtil.toObj(JsonUtil.toString(payConfigMap), Config.class);
        log.error("初始化的支付配置信息, config : {}", JsonUtil.toString(this.config));
    }
}